import datetime
from flask import Blueprint, request, jsonify, make_response, json
from app.utils import add_logo, get_logo_from_logotable, token_required, paginated_query, delete_record, single_record_query
from app import db
from app.doctor.models import Doctor  
from app.devices.models import Device
from app.rooms.models import Room
from app.doctor.schemas import doctor_schema  
from sqlalchemy import desc

doctor = Blueprint('doctor', __name__)

@doctor.route('/api/v1/add/doctor', methods=['POST'])
@token_required(1, 2, 3)
def create_doctor(current_user):
    try:
        doctor_dict = json.loads(request.form.get('formData'))
        if doctor_dict['device_name'] == "Device not available":
            doctor_dict['device_name'] = ""
        loaded_doctor_data = doctor_schema.load({
            'site_id': request.args.get('site_id'),
            'doctor_name': doctor_dict['doctor_name'].strip(),
            'device_id': str(doctor_dict['device_id']),
            'device_name': doctor_dict.get('device_name', ''),
            'gender': doctor_dict['gender'].capitalize(),
            'dob': doctor_dict.get('dob') or None, 
            'specialization': doctor_dict.get('specialization'),
            'email': doctor_dict.get('email'),
            'logo_id': add_logo(request.files.get('image')),
        })
        loaded_doctor_data.pop('device_name', None)
        db.session.add(Doctor(**loaded_doctor_data))
        db.session.commit()
        return make_response(jsonify({'message': 'Doctor added successfully', 'status': 'Success'}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, 'message': 'Failed to add Doctor', 'status': 'error'}), 500)


@doctor.route('/api/v1/get/doctor', methods=['GET'])
@token_required(1, 2, 3)
def get_doctor(current_user):
    try:
        doctor_query = Doctor.query.order_by(desc(Doctor.doctor_id))
        doctors_paginated, data = paginated_query(doctor_query, [Doctor.doctor_name], Doctor.site_id)
        serialized_doctors = []
        for doc in doctors_paginated:
            logo_data = get_logo_from_logotable(doc.logo_id) if doc.logo_id else ("", "")
            device_name = ""
            room_name = ""
            device = Device.query.filter_by(device_id=doc.device_id).first()
            if device:
                device_name = device.device_name
                room = Room.query.filter_by(id=device.room_id).first()
                if room:
                    room_name = room.room_name
            serialized_doctors.append({
                'id': doc.doctor_id,
                'site_id': doc.site_id,
                'doctor_name': doc.doctor_name,
                'specialization': doc.specialization,
                'device_id': doc.device_id,
                'room_name': room_name,
                'device_name': device_name,
                'gender': doc.gender,
                'dob': doc.dob.strftime('%Y-%m-%d') if doc.dob else None,
                'email': doc.email,
                'logo': logo_data[0],
                'logo_content_type': logo_data[1],
            })
        return make_response(jsonify({'doctors': serialized_doctors, 'data': data}), 200)
    except Exception as e:
        return make_response(jsonify({'message': f'Error: {str(e)}', 'status': 'error'}), 500)


@doctor.route('/api/v1/get/doctor_by_id/<int:id>', methods=['GET'])
@token_required(1, 2, 3)
def get_doctor_by_id(current_user, id):
    try:
        doctor = single_record_query(Doctor, Doctor.doctor_id, id, current_user)
        if doctor:
            logo_data = get_logo_from_logotable(doctor.logo_id) if doctor.logo_id else ("", "")
            device = Device.query.filter_by(device_id=doctor.device_id).first()
            device_name = device.device_name if device else None
            return make_response(jsonify({'doctor': {
                'id': doctor.doctor_id,
                'site_id': doctor.site_id,
                'doctor_name': doctor.doctor_name,
                'device_name': device_name,
                'specialization': doctor.specialization,
                'gender': doctor.gender,
                'dob': doctor.dob.strftime('%Y-%m-%d') if doctor.dob else None,
                'email': doctor.email
            }, "logo": {
                'logo': logo_data[0],
                'logo_content_type': logo_data[1],
            }}), 200)
        return make_response(jsonify({'message': 'Doctor not found', 'status': 'error'}), 404)
    except Exception as e:
        return make_response(jsonify({'message': f'Error: {e}', 'status': 'error'}), 500)


@doctor.route('/api/v1/update_doctor/<int:doctor_id>', methods=['PUT'])
@token_required(1, 2, 3)
def update_doctor(current_user, doctor_id):
    try:
        doctor_dict = json.loads(request.form.get('formData'))
        doctor_to_update = single_record_query(Doctor, Doctor.doctor_id, doctor_id, current_user)
        if not doctor_to_update:
            return make_response(jsonify({'message': 'Doctor not found', 'status': 'error'}), 404)
        if doctor_dict['device_name'] == "Device not available" or doctor_dict['device_name'] == None:
            doctor_dict['device_name'] = ""

        for key, value in doctor_schema.load({
            'site_id': request.args.get('site_id'),
            'doctor_name': doctor_dict['doctor_name'].strip(),
            'gender': doctor_dict.get('gender', doctor_to_update.gender).capitalize(),
            'dob': doctor_dict.get('dob', doctor_to_update.dob),
            'device_id': str(doctor_dict.get('device_id', doctor_to_update.device_id)),
            'specialization': doctor_dict.get('specialization', doctor_to_update.specialization),
            'email': doctor_dict.get('email', doctor_to_update.email),
            'logo_id': add_logo(request.files['image']) if 'image' in request.files else doctor_to_update.logo_id
        }).items():
            setattr(doctor_to_update, key, value)

        db.session.commit()
        return make_response(jsonify({'message': 'Doctor updated successfully', 'status': 'Success'}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, 'message': 'Failed to update Doctor', 'status': 'error'}), 500)


@doctor.route('/api/v1/delete-doctor/<id>', methods=['DELETE'])
@token_required(1, 2, 3)
def delete_doctor(current_user, id):
    return delete_record(Doctor, current_user, id, "Doctor", "doctor_id")
